function mults=getmult(saveirf1L,saveirf2L,STATE,sdata,realincomepercap)

% calculate govt spending multiplier as in caggiano et al EJ 2015 (Table 3)
% inputs: IRFs and level time series for federal spending and state income

% -------------------------------------------------------------------------
% IRFs: load irfi, created in getirfregimeiM.m. i=1 (boom, 2 (bust)
% matrices in irfi are saveirfiL in levels and saveirfi in logs
% dimensions reps=5000 x horizon=40 x nvar=4

% -------------------------------------------------------------------------
% time series for federal spending and state income to scale multipliers 
% (convert to $)
% load from data folder 
% time series are not complete for each state
% -------------------------------------------------------------------------

% order of variables (nvar)
% #1 Ramey shock
% #2 federal spending
% #3 income in other states
% #4 income in state i

% -------------------------------------------------------------------------
% LSP 4/18/17
% -------------------------------------------------------------------------


close all
clc

%% IRF ratio


% cumulate IRFs recession
irfy1 = (cumsum(saveirf1L(:,:,4),2));
irfg1 = (cumsum(saveirf1L(:,:,2),2));
mult1 = irfy1./irfg1;

% cumulate IRFs boom
irfy2 = (cumsum(saveirf2L(:,:,4),2));
irfg2 = (cumsum(saveirf2L(:,:,2),2));
mult2 = irfy2./irfg2;

mult(:,:,1) = mult1;
mult(:,:,2) = mult2;

%% convert to $: scale by average Y/G


%load spending
tmp=sdata;
GS=tmp(:,7); %Fed spending
%load state level real income per cap

statex=[nan(4,51);realincomepercap;nan(1,51)];

scalemat = statex(:,STATE)./GS;

% check min and max to see if Ramey Zubairy criticism applies
% seems to, the range is quite large
% min(min(scalemat))
% max(max(scalemat))

dens = nanmean(GS); 
% mean over time for each state, then median of states
nums = nanmean(statex(:,STATE));
% median of states, then mean over time
% nums = nanmean(nanmedian(statex'));

scale = nums/dens;

% scaled multipliers
mults = mult*scale;


% %% plot
% 
% % pick horizons you want in quarters
% ih = [4 8 12 16 20]; 
% 
% fprintf('\nGovernment spending multipliers\n');
% fprintf('%10s %10s %10s\n','Horizon','Regime 1','Regime 2');
% for i=1:numel(ih)
%     fprintf('%10.3f %10.3f %10.3f\n',ih(i),median(mults(:,ih(i),1),1),...
%         median(mults(:,ih(i),2),1));
% end
% fprintf('\n');
% 
% 
% names{1}='Multiplier Recession';
% names{2}='Multiplier Boom';
% tmp=prctile(mults,[50 16 84]);
% H=(3:39)';
% figure(1)
% for j=1:2
% ax(j) = subplot(1,2,j);
% plotx2(H,tmp(:,H,j)');
% title(names{j});
% end
% linkaxes(ax)
% 
% 
% save('mult')


% figure(1)
% plot([mult1s' mult2s'])
% title('Multipliers')
% xlabel('Horizon')
% legend('Recession','Boom')

% figure(2)
% plot([mult1' mult2'])
% title('Cumulative IRF ratio')
% xlabel('Horizon')
% legend('Recession','Boom')
% 
% figure(3)
% plot([irfy1' irfg1'])
% title('Cumulative IRFs')
% xlabel('Horizon')
% legend('Output','Govt spending')
