% tfpnews.m
%
% function that extract TFP news shock as in Barsky and Sims (JME 2011);
% then computes variance decompositions and impulse responses.
%
% Andre Kurmann, FRB; Last modified December 2012
%-----------------------------------------------------------------------

function alpha=getnews(b,vmat,nvars,nlags,KLbar,KUbar)

%companion matrix of demeaned VAR (=> don't include constant term)
M=zeros(nvars*nlags,nvars*nlags);			
M(1:nvars,:)=b(1:nvars*nlags,:)';
M(nvars+1:nvars*nlags,1:nvars*nlags-nvars)=eye(nvars*nlags-nvars);

% Extract impulse vectors 
%------------------------

%Lower triangular matrix s.t. vcv of fundamental shocks is I matrix = Cholesky decomp of vmat
Gtilde = chol(vmat)';

% compute VMA coeffs Bi
ei = zeros(nvars,1); 
ei(1,1) = 1;      %selection matrix; TFP needs to be ordered first in VAR

B = zeros(nvars,nvars,KUbar+1);
Bi = zeros(1,nvars,KUbar+1);
for l=0:KUbar;
    C=M^l;
    B(:,:,l+1) = C(1:nvars,1:nvars);
    Bi(:,:,l+1) = ei'*C(1:nvars,1:nvars); % Bi are VMA coeffs on how ith variable responds to different shocks l+1 periods back
                                       % Bi(:,:,l+1) is the coef on epsilon(t-l)
end

%compute V
V = zeros(nvars,nvars);
for l=0:KUbar;
    V = V + (KUbar+1-max(KLbar,l))*(Bi(:,:,l+1)*Gtilde)'*(Bi(:,:,l+1)*Gtilde);
end
    
VHat = V(2:nvars,2:nvars);

[eigenVector,eigenValue]=eig(VHat);               %eigenvectors have norm=1
lambda=[diag(eigenValue) seqa(1,1,nvars-1)];
order=sortrows(lambda,-1);     %sort eigenvalues in descending order
Vord=[];
for i=1:nvars-1;
    Vord=[Vord eigenVector(:,order(i,2))];    %reorder eigenvector according to ordered
end                                 %eigenvalues            
                          
%impulse vectors of the largest eigenvalues
gamma = zeros(nvars,1);
gammaHat=Vord(:,1); %VHat(:,1);  % use directely Vord matrix (AK)
gamma = [0;gammaHat];   %first element of gamma = 0 so as to implement assumption that news shock has no contemp effect on TFP (ordered 1st)
alpha=Gtilde*gamma;   %nvars x 1 matrix with alpha in the column 
